// SPDX-FileCopyrightText: 2022 Matteo Settenvini <matteo.settenvini@montecristosoftware.eu>
// SPDX-License-Identifier: AGPL-3.0-or-later

mod config;

use {
    self::config::Config,
    clap::{arg, command, ArgMatches, Command},
    reqwest::Url,
};

fn parse_args() -> ArgMatches {
    command!()
        .propagate_version(true)
        .subcommand_required(true)
        .arg_required_else_help(true)
        .subcommand(
            Command::new("init")
                .about("Authenticate against the provided NextCloud server")
                .arg(arg!(<server> "NextCloud server to connect to")),
        )
        .get_matches()
}

#[tokio::main(flavor = "multi_thread")]
async fn main() -> anyhow::Result<()> {
    let args = parse_args();
    let mut configuration = Config::new();

    match args.subcommand() {
        Some(("init", sub_matches)) => {
            let server = sub_matches
                .get_one::<String>("server")
                .expect("Mandatory parameter <server>");
            tokio::task::block_in_place(move || -> anyhow::Result<()> {
                configuration
                    .credentials
                    .add(Url::parse(server)?)
                    .expect("Unable to authenticate to NextCloud instance");
                Ok(())
            })?;
        }
        _ => unreachable!("Exhausted list of subcommands and subcommand_required prevents `None`"),
    };

    Ok(())
}
